-- Shader Selector for 0.9.x
-- by Phoenix Enero

function love.load()
	--
	print "\nLoaded Shader Selector by substitute541\n"

	shader = love.graphics.newShader("shader.glsl")

	mainCanvas = love.graphics.newCanvas()

	terrain = drawTerrain()
	terrainImage = love.graphics.newImage(terrain)
	normalImage = love.graphics.newImage(drawNormals(terrain))

	drawCanvas(mainCanvas)
end

local function clamp(v, M, m)
	return (v > M and M or v) < m and m or v
end

local function fractalNoise2(x, y, n, a, f)
	local ca = 1
	local cf = 1
	local val = 0
	local v = 0
	for _=1, n do
		val = val + (love.math.noise(x*cf, y*cf)*2-1)*ca
		v = v + ca
		ca = ca * a
		cf = cf * f
	end
	return val/v
end

function drawTerrain()
	local noisedata = love.image.newImageData(800, 600)
	local seed = math.random()*6000
	for y=1, 600 do
		for x=1, 800 do
			local val = fractalNoise2(x/150+seed, y/150+seed, 7, 0.4, 2.5)*0.5+0.5
			local val2 = fractalNoise2(x/300+seed, y/300+seed, 5, 0.4, 1.5)*0.5+0.5
			noisedata:setPixel(x-1, y-1, val*255, val2*255, 0, 255)
			--noisedata:setPixel(x-1, y-1, val*val2*255, val*val2*255, val*val2*255, 255)
		end
	end
	return noisedata
end

function drawNormals(data)
	local normals = love.image.newImageData(800, 600)
	local sqrt = math.sqrt
	local floor = math.floor
	local a = 50
	for y=1, 600-2 do
		for x=1, 800-2 do
			local s1, s2 = data:getPixel(x-1, y)
			local s3, s4 = data:getPixel(x+1, y)
			local v1 = s1*s2/65025
			local v2 = s3*s4/65025
			s1, s2 = data:getPixel(x, y-1)
			s3, s4 = data:getPixel(x, y+1)
			local v3 = s1*s2/65025
			local v4 = s3*s4/65025
			local s5, s6 = data:getPixel(x, y)
			local v5 = s5*s6/65025

			local xgrad = a*(v1-v2)
			local ygrad = a*(v3-v4)

			local d = sqrt(xgrad*xgrad+ygrad*ygrad+1)
			xgrad = xgrad/d
			ygrad = ygrad/d
			local z = 1/d

			normals:setPixel(x, y, floor(xgrad*128+128), floor(ygrad*128+128), z*255, 255)
		end
	end
	return normals
end

function drawCanvas(main)
	love.graphics.setShader()
	love.graphics.setCanvas(main)
	love.graphics.setBlendMode("alpha")
	love.graphics.setColor(255, 255, 255)

	love.graphics.draw(normalImage, 0, 0)

	love.graphics.setCanvas()
end

function love.mousepressed(x, y, btn)
	if btn == "l" then
		print(terrain:getPixel(x, y))
	end
end

function love.update(dt)
	love.window.setTitle("FPS: " .. love.timer.getFPS())
	shader:send("sundir", {0.5-love.mouse.getX()/800, 0.5-love.mouse.getY()/600})
	shader:send("colormap", terrainImage)
end

function love.draw()
	love.graphics.setShader(shader)
	love.graphics.draw(mainCanvas, 0, 0)
end

-- -- Note: y goes down, x goes right, z goes in
-- function makeNormals(imagedata)
-- 	local sqrt = math.sqrt
-- 	local normaldata = love.image.newImageData(imagedata:getWidth(), imagedata:getHeight())
-- 	for y=0, imagedata:getHeight()-2 do
-- 		for x=0, imagedata:getWidth()-2 do
-- 			local v0 = imagedata:getPixel(x, y)
-- 			local v1 = imagedata:getPixel(x+1, y)
-- 			local v2 = imagedata:getPixel(x, y+1)

-- 			-- changes
-- 			local dx = v0-v1
-- 			local dy = v0-v2
-- 			if math.abs(dy) <= 1 then dy = 0 end

-- 			-- normalize
-- 			local d = sqrt(dx*dx+dy*dy+1)
-- 			local nx = dx/d
-- 			local ny = dy/d
-- 			local nz = 1 - nx*nx - ny*ny
-- 			normaldata:setPixel(x, y, nx*128+128, ny*128+128, nz*128+128, 255)
-- 		end
-- 	end
-- 	return normaldata
-- end